//
//  AMSDKLogger.h
//  alimeeting
//
//  Created by siheng on 2019/12/17.
//  Copyright © 2019年 alimeeting. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class AMSDKLogger;

/**
 * 会议日志协议，可实现此协议处理会议日志信息
 */
@protocol AMSDKLoggerProtocol <NSObject>

@required

//! 输出info级别日志
+ (void)info:(NSString *)tag msg:(NSString *)msg;

//! 输出debug级别日志
+ (void)debug:(NSString *)tag msg:(NSString *)msg;

//! 输出warn级别日志
+ (void)warn:(NSString *)tag msg:(NSString *)msg;

//! 输出error级别日志
+ (void)error:(NSString *)tag msg:(NSString *)msg;

//! SLS Action日志
+ (void)action:(NSString *)module
         point:(NSString *)monitorPoint
        params:(NSDictionary * _Nullable)params;

//! SLS Event日志
+ (void)event:(NSString *)module
        point:(NSString *)monitorPoint
       params:(NSDictionary * _Nullable)params;

//! SLS Error日志
+ (void)error:(NSString *)module
        point:(NSString *)monitorPoint
       params:(NSDictionary * _Nullable)params;

@end

/**
 * 默认Logger实现，如果没有注入日志实现类则调用NSLog
 */
@interface AMSDKLogger : NSObject <AMSDKLoggerProtocol>

//! 注入日志类
+ (void)setLoggerPrinter:(Class<AMSDKLoggerProtocol>)printer;

@end

/**
 * 日志级别
 */
typedef NS_ENUM(NSInteger, AMSDKLoggingLevel) {
    //! Verbose
    AMSDKLoggingLevelVerbose    =  1 << 0,
    //! Info
    AMSDKLoggingLevelInfo       =  1 << 1,
    //! Warn
    AMSDKLoggingLevelWarn       =  1 << 2,
    //! Error
    AMSDKLoggingLevelError      =  1 << 3,
    //! Debug
    AMSDKLoggingLevelDebug      =  1 << 4,
};

#ifdef  __cplusplus
extern "C" {
#endif
    //! 获取当前日志所处文件名
    extern NSString* AMSDKLoggingFileName(const char *fileName);
    //! 输出日志方法
    extern void AMSDKLogging(AMSDKLoggingLevel level, NSString *tag, NSString *string);
    
#ifdef  __cplusplus
}
#endif

#define AMSDKLoggingString(format, ...)                         \
[NSString stringWithFormat:@"(%@:%d %s): " format,              \
AMSDKLoggingFileName(__FILE__),                                 \
__LINE__,                                                       \
__FUNCTION__,                                                   \
##__VA_ARGS__                                                   \
]

#define AMSDKLoggingFormat(level, tag, format, ...)                  \
do {                                                                 \
AMSDKLogging(level, tag, AMSDKLoggingString(format, ##__VA_ARGS__)); \
} while(NO)

#define AMSDKLoggingVerbose(tag, format, ...)                   \
AMSDKLoggingFormat(AMSDKLoggingLevelVerbose, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingInfo(tag, format, ...)                      \
AMSDKLoggingFormat(AMSDKLoggingLevelInfo, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingWarn(tag, format, ...)                      \
AMSDKLoggingFormat(AMSDKLoggingLevelWarn, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingError(tag, format, ...)                     \
AMSDKLoggingFormat(AMSDKLoggingLevelError, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingDebug(tag, format, ...)                     \
AMSDKLoggingFormat(AMSDKLoggingLevelDebug, tag, format, ##__VA_ARGS__)

NS_ASSUME_NONNULL_END
