//
//  AMUiSDKMeetingCallBack.h
//  alimeetingUiSDK
//
//  Created by erqin on 2019/12/5.
//  Copyright © 2019 Alibaba. All rights reserved.
//

#ifndef AMUISDKMeetingCallBack_h
#define AMUISDKMeetingCallBack_h

#import <Foundation/Foundation.h>


typedef NS_ENUM(NSInteger, AMUISDKMeetingErrorCode)
{
    //! 非法参数，接口调用参数不正确
    AMUISDKMeetingErrorCodeIllegalParameters = 0,
    //! 会议初始化失败，如无法获取会议信息，拉去配置
    AMUISDKMeetingErrorCodeInitializedFailed = 1,
    //! 设备无权限
    AMUISDKMeetingErrorCodeDevicePermissionNotGranted = 2,
    //! 无麦克风设备
    AMUISDKMeetingErrorCodeNoDevice = 3,
    //! 无法连接媒体服务
    AMUISDKMeetingErrorCodeMediaServerConnectFailed = 4,
    //! 会议并发满了
    AMUISDKMeetingErrorCodeRoomFull = 5,
    //! 入会超时
    AMUISDKMeetingErrorCodeJoinMeetingTimeout = 6,
    //!网络不可用
    AMUISDKMeetingErrorCodeNetworkUnAvailable = 7,
    //!媒体协商失败
    AMUISDKMeetingErrorCodeMediaNegotiationFailed = 8
};

typedef NS_ENUM(NSInteger, AMUISDKMeetingFinishCode)
{
    //!注定退出会议
    AMUISDKMeetingFinishCodeLeaveBySelf = 0,
    //!被动退出会议
    AMUISDKMeetingFinishCodeLeaveByKick = 1,
    //!挂断所有人
    AMUISDKMeetingFinishCodeLeaveByHangupAll = 2,
    //!同一个UserID的用户进入了会议
    AMUISDKMeetingFinishCodeOtherClientJoined = 3,
    //!会议评价完成
    AMUISDKMeetingFinishCodeMeetingRateComplete = 4,
    //!系统挂起
    AMUISDKMeetingFinishCodeSystemWillSleep = 5
};

/**
 * Meeting User状态事件
 */
typedef NS_ENUM(NSInteger, AMSDKUserStatusEvent) {
    //! 成员上线
    AMSDKUserStatusEventOnline                 =   0,
    //! 成员离线
    AMSDKUserStatusEventOffline                =   1,
    //!静音
    AMSDKUserStatusEventAudioMute              =   2,
    //!取消静音
    AMSDKUserStatusEventAudioUnMute            =   3,
    //!禁视频
    AMSDKUserStatusEventVideoMute              =   4,
    //!打开视频
    AMSDKUserStatusEventVideoUnMute            =   5,
    //!说话中
    AMSDKUserStatusEventStartTalking           =   6,
    //!结束说话
    AMSDKUserStatusEventStopTalking            =   7,
    //!主讲人切换
    AMSDKUserStatusEventMainSpeaker            =   8,
};

@class AMSDKMeetingUser;

@protocol AMUISDKMeetingEventDelegate <NSObject>
//必选实现回调
@required
//已成功进入会议
- (void) onMeetingJoined;

// 会议正常结束
- (void) onMeetingFinish:(AMUISDKMeetingFinishCode) code reason:(NSString*) reason;

// 入会异常
- (void) onError:(AMUISDKMeetingErrorCode)code reason:(NSString*) reason;

//可选实现回调
@optional

// 邀请成员回调
- (void) onInviteClicked:(NSString*) reason;

//成员加入/离开消息回调
- (void) onMeetingUserStatusChange:(AMSDKMeetingUser *)meetingUser event:(AMSDKUserStatusEvent)event;
@end

#endif /* AMUiSDKMeetingCallBack_h */
