/*
 原生微服务应用管理器托管客户端对接协议实现
 */
#ifndef __H_ALILANG_NATIVE_MESSAGE_PROTOCOL_H__
#define __H_ALILANG_NATIVE_MESSAGE_PROTOCOL_H__

// 跨平台C接口基本数据类型统一定义规范
#define S8    signed char
#define U16   unsigned short
#define S16   signed short
#define U32   unsigned int
#define S32   signed int
#define BOOL  signed char
#define CHAR  char
#define PCHAR const char *
#define ERROR const char *

// 基础常量的定义
#define OK    ((const char *)0)
#define MAX_MESSAGE_SIZE    1 * 1024 * 1024

/**
 * 原生微服务客户端初始化并监听外部请求信息
 * @param pcAppId  当前应用标识（必须要注册表中已经存在的目录才是合法的应用标识名称）
 * @param i_pcServerPipeFile 服务端的管道通信句柄
 * @return
 */
extern "C" ERROR LIBAPPMNGCLIENT_Run(PCHAR i_pcAppId, PCHAR i_pcServerPipeFile);

/**
 * 目标服务的远程方法调用（原生微服务之间的消息请求响应过程中的处理逻辑实现）
 * @param i_pcInputJsonStrMsgBody
 * @param o_pcOutputJsonStrMsgBodyBuffer
 * @param o_u32OutputJsonStrMsgBodyBufferSize
 * @return
 */
extern "C" ERROR LIBAPPMNGCLIENT_Call(PCHAR i_pcInputJsonStrMsgBody,
                                      PCHAR o_pcOutputJsonStrMsgBodyBuffer,
                                      U32 o_u32OutputJsonStrMsgBodyBufferSize, bool shouldRecvMessage);

// C++防御式编程设计要点使用约束：
// 1. 所有的函数定义返回值必须要是ERROR类型。
// 2. C风格定义动态库导出接口，C++用于定义内部接口以及代码内部实现，资源管理全部用智能指针进行自动封装（避免业务代码中出现使用free或delete场景）。
// 3. 错误编码以字符串常量定义，全局的统一定义按需进行引用报错处理，导出错误码的常量字符串定义而不是数字，提供统一的跨平台日志逻辑实现
#define XCALL(f) {ERROR ret = (f); if(ret){__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,ret,"");return ERROR_XCALL;}}
#define XCALLS(f, format, args...) {ERROR ret = (f); if(ret){__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,ret,format,##args);return ERROR_XCALL;}}
#define XASSERT(expr) {if(!(expr)){__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,ERROR_XASSERT,"");return ERROR_XASSERT;}}
#define XASSERTS(expr, format, args...) {if(!(expr)){__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,ERROR_XASSERT,format,##args);return ERROR_XASSERT;}}
#define XRETURN(ErrorCode) {if(ErrorCode){__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,ERROR_XRETURN,"");} return ErrorCode;}
#define XLOG(format, args...) {__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,"OK",format,##args);}
#define XDEBUG(format, args...) {__LIBAPPMNGCLIENT_PrintLog__(__FILE__,__LINE__,"DEBUG",format,##args);}
// 给防御式错误处理机制内部使用的跨平台日志打印函数
extern "C" ERROR __LIBAPPMNGCLIENT_PrintLog__(PCHAR i_pcFile, U32 i_u32Line, ERROR i_pcErrorCode,
                                              const char *format, ...);

// 错误码常量值含义的统一定义（用ASCII的C标识符的字符串常量取代传统的数字定义增强语义性，将错误值的多语言翻译放在服务端统一管理日志分析的时候查询）

// 约定XYZ开头的为框架定义的错误码
#define ERROR_XASSERT "ERROR_XASSERT"   // 断言错误
#define ERROR_XCALL   "ERROR_XCALL"     // 函数调用错误
#define ERROR_XRETURN "ERROR_XRETURN"   // 函数返回错误

// 约定LIB开头的为第三方动态库统一注册的错误码定义（类似npm包名需要统一注册管理）
#define ERROR_LIBAPPMNGCLIENT_OnlySupportSingletonProcessInstance "ERROR_LIBAPPMNGCLIENT_OnlySupportSingletonProcessInstance"
#define ERROR_LIBAPPMNGCLIENT_CallbackProcessMessageThrowException "ERROR_LIBAPPMNGCLIENT_CallbackProcessMessageThrowException"
#define ERROR_LIBAPPMNGCLIENT_InvalidMessageJsonFormat "ERROR_LIBAPPMNGCLIENT_InvalidMessageJsonFormat"
#define ERROR_LIBAPPMNGCLIENT_NotFoundApiImplementation "ERROR_LIBAPPMNGCLIENT_NotFoundApiImplementation"

#endif
