#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/ucred.h>
#include <sys/stat.h>
#include <stdint.h>
#include <string>
#import <string>
#import <Foundation/Foundation.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <istream>
#include <iostream>
#include <utility>
#include <thread>
#include <chrono>
#include <functional>
#include <atomic>
#include <sys/time.h>
#include "json.hpp"
#include "alilang_native_message_protocol_darwin.h"
#import "AMClientConfigManager.h"

namespace OS {
    ERROR RunSingleProcessInstance(PCHAR i_pcAppId);

    ERROR PrintLog(PCHAR i_pcLogInfo);

    // 定义跨平台的管道通信类的封装实现（隔离掉平台差异性代码，以及进行管道资源的智能指针管理）
    class PipeIpc {
    private:
        S32 client = -1;
        std::string sServerPipeFile;
    public:
        PipeIpc(const std::string &i_sSourceAppId);

        ~PipeIpc();

        ERROR ConnectRemoteServer(PCHAR pcJsonStrMsgBody);

        // 接收消息
        ERROR RecvMsg(PCHAR o_pcReceivedJsonStrMsgBody, U32 o_u32ReceivedJsonStrMsgBodyBufferSize);

        // 发送消息
        ERROR SendMsg(PCHAR pcJsonStrMsgBody);
    };
}

ERROR OS::PrintLog(PCHAR i_pcLogInfo) {
    NSLog(@"%s", i_pcLogInfo); // 日志打印到控制台一份信息
    XRETURN(OK);
}

OS::PipeIpc::PipeIpc(const std::string &i_sServerPipeFile) {
    this->sServerPipeFile = i_sServerPipeFile;
}

OS::PipeIpc::~PipeIpc() {
    if (this->client >= 0) {
        close(this->client);
        this->client = -1;
    }
}

ERROR OS::PipeIpc::ConnectRemoteServer(PCHAR pcJsonStrMsgBody) {
    // 连接原生应用管理器（进行消息请求转发路由处理）
    struct sockaddr_un addr = {0};
    memset(&addr, 0, sizeof(addr));
    addr.sun_family = AF_UNIX;
    strncpy(addr.sun_path, this->sServerPipeFile.c_str(), sizeof(addr.sun_path) - 1);
    this->client = socket(AF_UNIX, SOCK_STREAM, 0);
    XASSERT(this->client >= 0);

    int ret = connect(this->client, (struct sockaddr *) &addr, sizeof(struct sockaddr_un));
    XASSERTS(ret >= 0 , "connect error %s", strerror(errno));
    
    if(ret < 0 )
    {
        printf("#Message#%s#Message#",pcJsonStrMsgBody);
        fflush(stdout);
    }

    XRETURN(OK);
}

// 接收消息
ERROR OS::PipeIpc::RecvMsg(PCHAR o_pcReceivedJsonStrMsgBody, U32 o_u32ReceivedJsonStrMsgBodyBufferSize) {
    S32 client = this->client;
    XASSERT(client >= 0);

    ssize_t n = 0;

    // 先读取消息头固定字节内容
    char acHeaderBuf[9]; // 消息内容的16进制字符串（8个字符补零），解析出来的整数就是负载utf8字符串内容buffer长度（不含零字节）
    memset(acHeaderBuf, 0, sizeof(acHeaderBuf));
    n = recv(client, acHeaderBuf, 8, 0);
    XASSERTS(n == 8, "received msg header size = %d, header size = %d", n, 8);
    char *str;
    U32 u32MsgPayloadSize = (int) strtol(acHeaderBuf, &str, 16);//十六进制

    // 循环读取数据消息体内容的数据（读取完成就结束，根据短消息通信协议每次短消息请求仅仅发送一个合法的消息，对于多发送的内容视为非法通信协议需要禁止掉，进而简化通信算法的实现逻辑健壮性）
    // 在读取过程中可能会出现中断现象需要返回错误值结束消息接收过程
    ssize_t iReadSize = 0;
    for (; iReadSize < u32MsgPayloadSize; iReadSize += n) {
        n = recv(client,
                 (void *) (o_pcReceivedJsonStrMsgBody + iReadSize),
                 (size_t) (u32MsgPayloadSize - iReadSize), 0);
        if (n <= 0) {
            break;
        }
    }
    XASSERTS(iReadSize > 0 && iReadSize == u32MsgPayloadSize,
             "received msg size = %d, length=%d", iReadSize, u32MsgPayloadSize);
    *((char *) o_pcReceivedJsonStrMsgBody + u32MsgPayloadSize) = 0; // 字符串终止符

//    XLOG("receive ipc message, protocol: %d, version: %d \r\n", tMsgHeader.protocol, tMsgHeader.version);
    XRETURN(OK);
}

// 发送消息
ERROR OS::PipeIpc::SendMsg(PCHAR pcJsonStrMsgBody) {
    S32 client = this->client;
    XASSERT(client >= 0);

    U32 u32MsgPayloadSize = strlen(pcJsonStrMsgBody);
    // TODO 返回消息内容过长错误
    XASSERT(u32MsgPayloadSize > 0 && u32MsgPayloadSize < MAX_MESSAGE_SIZE);

    char acHeaderBuf[9]; // 消息内容的16进制字符串（8个字符补零），解析出来的整数就是负载utf8字符串内容buffer长度（不含零字节）
    memset(acHeaderBuf, 0, sizeof(acHeaderBuf));
    sprintf(acHeaderBuf, "%08x", u32MsgPayloadSize);

    // 先发送消息头部
    ssize_t n = 0;
    n = send(client, acHeaderBuf, 8, 0);
    XASSERTS(n == 8, "send error %s", strerror(errno));

    // 再循环发送消息内容（数据量过大的时候可能会超出系统缓存的空间大小需要进行分批发送处理）
    int iSentSize = 0;
    for (; iSentSize < u32MsgPayloadSize; iSentSize += n) {
        n = send(client, pcJsonStrMsgBody + iSentSize, u32MsgPayloadSize - iSentSize, 0);
        XASSERTS(n > 0, "send error %s", strerror(errno));
    }
    XASSERTS(iSentSize == u32MsgPayloadSize,
             "send size = %d, content size = %d, send error %s\r\n",
             iSentSize, u32MsgPayloadSize, strerror(errno));

    // TODO 如何处理超时问题？？？如何处理发送过程中的异常情况？？？
    XRETURN(OK);
}

using json = nlohmann::json;

static struct {
    bool bIsInitialized;
    std::string sSourceAppId;
    std::string sServerPipeFile;
} s_tAppMng = {.bIsInitialized=false};

// 错误日志信息的统一响应处理
ERROR __LIBAPPMNGCLIENT_PrintLog__(PCHAR i_pcFile, U32 i_u32Line, ERROR i_pcErrorCode,
                                   const char *format, ...) {
    char acBuffer[1024 * 8];
    char acOut[1024 * 10]; // TODO 改进调试打印日志信息
    va_list args;
    va_start(args, format);
    vsprintf(acBuffer, format, args);
    va_end(args);
    if (!i_pcErrorCode) {
        i_pcErrorCode = "";
    }

    time_t now = time(NULL);
    struct tm *t = NULL;
    t = localtime(&now); //获取当地时间

    struct timeval tv;
    gettimeofday(&tv, NULL);

    // TODO 毫秒时间戳、当前线程ID（最好可以根据线程ID得到线程回调函数名称）、进程PID（多进程日志混写必须）
    sprintf(acOut, "[%04d-%02d-%02d %02d:%02d:%02d.%03d][%s][%u][%s][%s:%d]%s\r\n",
            t->tm_year - 100 + 2000, t->tm_mon + 1, t->tm_mday,
            t->tm_hour, t->tm_min, t->tm_sec, tv.tv_usec / 1000, // 显示毫秒时间戳
            s_tAppMng.sSourceAppId.c_str(), // 注册的应用名（取代进程PID）
            (pid_t) (uintptr_t) pthread_self(), // 当前线程ID
            i_pcErrorCode, i_pcFile, i_u32Line, acBuffer);

    try {
        static U32 s_u32TotalSavedSize = 0;
        s_u32TotalSavedSize += strlen(acOut);
        if (s_u32TotalSavedSize <= 30 * 1024 * 1024) { // 最大缓存30MB的调试日志记录文件禁止无限制的存储
            std::ofstream out(s_tAppMng.sSourceAppId + ".log", std::ios::app);
            if (out.is_open()) {
                out << acOut;
                out.close();
            }
        } else {
            s_u32TotalSavedSize = 0;
            std::ofstream out(s_tAppMng.sSourceAppId + ".log", std::ios::trunc);
            if (out.is_open()) {
                out << acOut;
                out.close();
            }
        }

        fprintf(stdout, "%s", acOut);
        fflush(stdout);
    } catch (...) {
        // ignore error 日志写入失败不能影响正常的业务执行过程
    }

    return OK; // 防止递归此处直接return
}

/**
 * 模块初始化
 * @param i_pcAppId    当前应用标识
 * @param i_pcServerPipeFile 服务端的管道通信句柄
 * @return
 */
ERROR __LIBAPPMNGCLIENT_Run__(PCHAR i_pcAppId, PCHAR i_pcServerPipeFile) {
    XASSERT(i_pcAppId && i_pcServerPipeFile);

    static char acAppId[1024]; // 拷贝一个副本，避免应用在长时间运行过程中释放成为野指针，导致mac端退出时候错误打印引用野指针出现crash。
    static char acServerPipeFile[1024]; // 拷贝一个副本，避免应用在长时间运行过程中释放成为野指针，导致mac端退出时候错误打印引用野指针出现crash。
    memset(acAppId, 0, sizeof(acAppId));
    memset(acServerPipeFile, 0, sizeof(acServerPipeFile));
    strncpy(acAppId, i_pcAppId, sizeof(acAppId) - 1);
    strncpy(acServerPipeFile, i_pcServerPipeFile, sizeof(acServerPipeFile) - 1);
    s_tAppMng.sSourceAppId = acAppId;
    s_tAppMng.sServerPipeFile = acServerPipeFile;

    // 如果不设置回调就补一个缺省回调，确保线程监听逻辑可以正常工作，用于与应用管理器状态同步表示进程已经正常启动完成。
    s_tAppMng.bIsInitialized = true;
    XRETURN(OK);
}

ERROR LIBAPPMNGCLIENT_Run(PCHAR i_pcAppId, PCHAR i_pcServerPipeFile) {
    // 关键业务流程一旦运行异常必须要退出当前主进程确保进程状态一致性
    try {
        __LIBAPPMNGCLIENT_Run__(i_pcAppId, i_pcServerPipeFile);
    } catch (...) {
        // ignore error
    }
    XRETURN(OK);
}

// 客户端主动发送消息给目标应用服务一般是事件通知消息类型（P2P的消息管控处理，需要导出给原生应用使用）
ERROR LIBAPPMNGCLIENT_Call(PCHAR i_pcInputJsonStrMsgBody,
                           PCHAR o_pcOutputJsonStrMsgBodyBuffer,
                           U32 o_u32OutputJsonStrMsgBodyBufferSize, bool shouldRecvMessage) {

    if([[AMClientConfigManager sharedInstance] isStaff])
    {
        // JSON解析出发送消息的内容结构体信息，验证参数schema格式，取目标应用注册表中的地址信息，发送请求消息，接收回应消息，返回给应用一个消息对象（应用要给一个buffer传递输出字符串内容）
        json in = json::parse(std::string(i_pcInputJsonStrMsgBody));
        // TODO 验证输入消息JSON schema格式的合法性
        
        // 连接原生应用管理器（进行消息请求转发路由处理）
        std::unique_ptr<OS::PipeIpc> uphClient(new OS::PipeIpc(s_tAppMng.sServerPipeFile.c_str()));
        XCALL(uphClient->ConnectRemoteServer(i_pcInputJsonStrMsgBody))
        
        // 发送请求消息
        XCALL(uphClient->SendMsg(i_pcInputJsonStrMsgBody));
        
        // 接收回应消息
        if(shouldRecvMessage)
        {
            XCALL(uphClient->RecvMsg(o_pcOutputJsonStrMsgBodyBuffer, o_u32OutputJsonStrMsgBodyBufferSize));
        }

    }

    XRETURN(OK);
}
